// ---------------------------------------------------------------------
/*
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

 \file	GHDSS_MathMatDouble.h
 \brief	\~English Header file for matrix operation[type double]. (GHDSS System)
 \brief	\~Japanese s񉉎Ẑ߂̃wb_t@C(GHDSS System)
 \author	Sachiko Suzuki
 \date	27 Jun 2008
 \version	$Id: GHDSS_MathMatDouble.h 456 2009-04-09 02:24:03Z suzuki $
*/

// ---------------------------------------------------------------------
#ifndef GHDSS_MathMatDouble_h_defined
#define GHDSS_MathMatDouble_h_defined
// ---------------------------------------------------------------------
#include	"MICARYHA_struct.h"
#include "MICARY_struct.h"
#include "MICARY_GHDSS_struct.h"
#include <stdlib.h>

#define __MAX_FLOAT_VALUE 3.0428E+038
#define __MIN_FLOAT_VALUE 1.1754E-038
#define __MAX_DOUBLE_VALUE 1.79769313E+308
#define __MIN_DOUBLE_VALUE 2.22507385E-308
// ---------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif

typedef struct 
{
    /// real part
    double re;
    /// imaginary part
    double im;
} DA_Complex;
/**
 Complex array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	DA_Complex *	da_cmpValue; ///<\~Japanese HA_Complexz
	HA_Num	ha_numValue; ///<\~Japanese z̃TCY
} MICARY_dComplexArray;

/**
 Spectrum struct (Double Complex)
 \author	Sachiko Suzuki
 \date 01 Jul 2008
*/
typedef struct{
	MICARY_dComplexArray	micary_cmpaSpec; ///<\~Japanese XyNg(Complex) 
	HA_Num	ha_numCh; ///<\~Japanese `l
	HA_Integer	ha_iLowerFreq; ///<\~Japanese g[Line]
	HA_Integer	ha_iUpperFreq; ///<\~Japanese g[Line]
} MICARY_dComplexSpecArray;


// ---------------------------------------------------------------------
/**
 \brief \~English It allocates complex array
 \brief \~Japanese HA_Complexz̗̈mۂ
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void dComplexArrayAlloc(MICARY_dComplexArray * array, int iArraySize);

/**
 \brief \~English It frees complex array
 \brief \~Japanese HA_Complexz 
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
*/
void dComplexArrayFree(MICARY_dComplexArray * array);

/**
 \brief \~English It initializes complex array
 \brief \~Japanese Complexz̏(0)
 \param array \~English It's complex array
 \~Japanese Complexz܂ލ\
*/

void dComplexArrayInit(MICARY_dComplexArray * array) ;

// 2008/07/01 added by S.Suzuki
/**
 \brief \~English It allocates complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈mۂ
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
*/
void AllocdComplexSpecArray(MICARY_dComplexSpecArray *pcsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex);

/**
 \brief \~English It frees complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈J
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
*/
void FreedComplexSpecArray(MICARY_dComplexSpecArray *pcsSpec);

/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductdfCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_ComplexArray caInMat2,int g2,int r2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductfdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductdffCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
									MICARY_ComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductfddCompMat1(MICARY_ComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2) ;

// ---------------------------------------------------------------------

/*
 \brief \~English It conjugates complex matrix.
 \brief \~Japanese 𕡑f
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
*/
int ConjdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat);

// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int DiagdCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat, int g , int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int DiagdfCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat, int g , int r) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int ConjTransdCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int ConjTransdfCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,int g,int r) ;
// ---------------------------------------------------------------------

/*
 \brief \~English It makes unit matrix.
 \brief \~Japanese Pʍs̍쐬
 \param g \~English It's the complex matrix struct
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int EyedCompMat1(MICARY_dComplexArray *pcaOutMat,int g,int r);

int EyedCompMat1v2(MICARY_dComplexArray *caOutMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It transrates complex matrix.
 \brief \~Japanese f̓]u
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/

int TransdCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It adds complex matrix.
 \brief \~Japanese fŝZ
 \param pcaOutMat1 \~English It's the complex array struct.
 \~Japanese o͕fs1
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SumdCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It adds complex matrix.
 \brief \~Japanese fŝZ
 \param pcaOutMat1 \~English It's the complex array struct.
 \~Japanese o͕fs1
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SumffdCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It substracts complex matrix.
 \brief \~Japanese fŝЂZ
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SubdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It substracts complex matrix.
 \brief \~Japanese fŝЂZ
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SubfdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It multiplicates complex matrix.
 \brief \~Japanese fs̒萔{
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs
 \param a \~English It's the coeficient value.
 \~Japanese Wl
 
*/

int MultipldCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat,double a) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It multiplicates complex matrix[float * double].
 \brief \~Japanese fs̒萔{
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs
 \param a \~English It's the coeficient value.
 \~Japanese Wl
 
*/
int MultiplfdCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat,double a) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It computes a dot product of a conjugated vector with another vector(0:false 1:true).
 \brief \~Japanese dot|Z(0:G[ 1:I)
 \param pcaOut \~English It's the complex  vector.
 \~Japanese o͕fxNg
 \param caIn1 \~English It's the input vector1.
 \~Japanese ͕fxNg1
 \param caIn2 \~English It's the input vector2.
 \~Japanese ͕fxNg2
*/
int DotcdCompMat1(MICARY_dComplexArray *pcaOut ,MICARY_dComplexArray caIn1 ,MICARY_dComplexArray caIn2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It calculates abs complex matrix.
 \brief \~Japanese s̐ΒľvZ
 \param pcaOutMat \~English It's the complex  matrix.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
int AbsdCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It calculates abs complex matrix.
 \brief \~Japanese s̐ΒľvZ
 \param pcaOutMat \~English It's the complex  matrix.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
int AbsdfCompMat1(MICARY_dComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It calculates max value.
 \brief \~Japanese s(real part)̍őľvZ
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
double MaxdCompMat1(MICARY_dComplexArray caInMat) ;

/*
 \brief \~English It devids complex matrix.
 \brief \~Japanese fs̊Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int DividedCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2) ;

/*
 \brief \~English It devids complex matrix.
 \brief \~Japanese fs̕
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs
*/
int SqrtdCompMat1(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1) ;

#ifdef __cplusplus
}
#endif
#endif


